uniform sampler2D iChannel0;
uniform sampler2D iChannel1;
uniform sampler2D iChannel2;
uniform sampler2D iChannel3;
uniform float iTime;
uniform vec4 iMouse;
uniform vec2 iResolution;
out vec4 C;
#define MAX_STEPS 200
#define MAX_DIST 150.
#define SURF_DIST .01
#define EPSILON .01
#define PI 3.14159265359


struct  Surface {
    float d;
    vec3 color;
};


const vec3 COLOR_BACKGROUND = vec3(0.25, 0.2, 0.22);

mat4 rotX (in float angle)
{
    float rad = radians (angle);
    float c = cos (rad);
    float s = sin (rad);

    mat4 mat = mat4 (vec4 (1.0, 0.0, 0.0, 0.0),
                     vec4 (0.0,   c,   s, 0.0),
                     vec4 (0.0,  -s,   c, 0.0),
                     vec4 (0.0, 0.0, 0.0, 1.0));

    return mat;
}

mat4 rotY (in float angle)
{
    float rad = radians (angle);
    float c = cos (rad);
    float s = sin (rad);

    mat4 mat = mat4 (vec4 (  c, 0.0,  -s, 0.0),
                     vec4 (0.0, 1.0, 0.0, 0.0),
                     vec4 (  s, 0.0,   c, 0.0),
                     vec4 (0.0, 0.0, 0.0, 1.0));

    return mat;
}

mat4 rotZ (in float angle)
{
    float rad = radians (angle);
    float c = cos (rad);
    float s = sin (rad);

    mat4 mat = mat4 (vec4 (  c,   s, 0.0, 0.0),
                     vec4 ( -s,   c, 0.0, 0.0),
                     vec4 (0.0, 0.0, 1.0, 0.0),
                     vec4 (0.0, 0.0, 0.0, 1.0));

    return mat;
}

// Tarefa

//Operaçoes  de translação

// Operações de escala

vec3 opTransf (vec3 p, mat4 m)
{
    return vec4 (m * vec4 (p, 1.)).xyz;
}

float sphereDist(vec3 p, vec3 ce, float r)
{

      vec3 c = p-ce;
      float rn = r +2.7*pow((0.5+0.5*sin(2.0*PI*iTime*0.75+c.y/15.0)),4.0);
      c.z*=1.25 -c.y/40.0;
       c.y-= abs(c.x)*sqrt((18.0-abs(c.x))/15.0);
       return (length(c)-rn);
}

float cylinderDist(vec3 p, vec3 ce,float r)
{
    vec2 c = (p-ce).xz;
    return (length(c)-r);
}

float planeDist(vec3 p, vec3 Normal,float D)
{
    return p.x*Normal.x + p.y*Normal.y + p.z*Normal.z -D;
}

Surface minS(Surface s1, Surface s2)
{

    return (s1.d<s2.d)? s1:s2;
}


Surface getDist(vec3 p)
{
    Surface heart;
    heart.d = sphereDist(p,vec3(0.0,10,20.0),10.0);
    heart.color = vec3 (0.9,0.1,0.15);
    Surface plane;
    plane.d = planeDist(p,vec3 (0.0,1.0,0.0),0.0);
    plane.color = vec3 (0.8);
    Surface r= minS(heart,plane);
    //float dc = cylinderDist(p,vec3(-15.0,10,5.0),3.0);
    //d =min(d,dc);
    return r;
}

Surface rayMarching(vec3 Cam, vec3 rd)
{
    float d0 = 0.0;
    vec3 pi = Cam +d0*rd;
    Surface s = getDist(pi);
    int i=0;
    while ((i<MAX_STEPS)&&(s.d>SURF_DIST)&&(d0<MAX_DIST))
    {
        d0+=s.d;
        pi = Cam+d0*rd;
        s=getDist(pi);
        i++;
    }
    if(i==MAX_STEPS) d0=MAX_DIST+1.0;
    s.d=d0;
    return s;
}

vec3 estimateNormal(vec3 p)
{
    Surface sd= getDist(p);

    float dx =  getDist(vec3(p.x+EPSILON,p.y,p.z)).d-sd.d;
    float dy = getDist(vec3(p.x,p.y+EPSILON,p.z)).d-sd.d;
    float dz = getDist(vec3(p.x,p.y,p.z+EPSILON)).d-sd.d;
    return normalize(vec3(dx,dy,dz));
}




vec3  getLight(vec3 p,vec3 CamPos, Surface s) {
    vec3 lightPos =vec3(20.0,30.0,-20.0);
    vec3 lightColor = vec3 (1.0);
    float w=2.0*PI*iTime*0.1;
    lightPos.xz+=0.25*length(lightPos-p)*vec2(cos(w),sin(w))*0.5;
    float ka = 0.3;
    vec3 ld = lightPos-p;
    float fat = 1/length(ld);
    vec3 l = fat*ld;
    vec3 n = estimateNormal(p);
    float dif = dot(n, l);

    vec3 col= s.color*ka;
    float kd = 0.35;
    col+= s.color*kd*dif*fat*20.0;
    float ks =0.35;
    vec3 eye = normalize(p-CamPos);
    vec3 R = normalize(reflect(n,l));
    float phi=dot(eye,R);
    if(phi>0.0)
        col+= ks*lightColor*pow(phi,5.0);
    Surface sd = rayMarching(p+200.0*n*SURF_DIST, l);
    if(sd.d<length(lightPos-p)) col *= .25;

    return col;
}

mat3 SetCamera(vec3 Look_at, vec3 CamPos)
{
    vec3 cd =normalize(Look_at-CamPos);
    vec3 cv = cross(vec3(0.0,1.0,0.0),cd);
    vec3 cu = cross(cd,cv);
    return mat3(cv,cu,cd);
}


void main()
{
    vec2 p = (gl_FragCoord.xy-0.5*iResolution)/iResolution.y;


    vec3 rd = normalize(vec3(p,1.2));
    vec3 col = mix(vec3 (.01,0.1,0.1),COLOR_BACKGROUND,-p.y*0.2);
    vec2 mo = iMouse.xy/iResolution.xy;

    float am = mix(0.0,90.0,mo.x*iMouse.z);
    float bm = mix(0.0,90.0,mo.y*iMouse.z);
    mat4 Mry = rotY(am);
    mat4 Mrx = rotX(bm);
    mat4 M = Mrx*Mry;
    vec3 Cam= vec3 (0.0,10.0,-35.0);
    Cam=opTransf(Cam,M);

    vec3 Target = vec3 (0.0,10,20);
    mat3 Mc = SetCamera(Target,Cam);
    rd = Mc*rd;

    Surface s = rayMarching(Cam,rd);
    if(s.d<MAX_DIST)
    {
        vec3 po= Cam+s.d*rd;
         col= getLight(po,Cam,s);
    }

    
    //col = mix(COLOR_BACKGROUND,col,f);
    C = vec4( col, 1.0 );
}

